/* vc_base.h (Vertex cover problem)

   Ambros Marzetta 94-11-10

   For more information see our WWW page at
   
         http://wwwjn.inf.ethz.ch/ambros/zram.html
	 
 * $Source$
 * $Revision: 2732 $
 * $Date: 2000-11-15 17:14:52 +0100 (Wed, 15 Nov 2000) $
 * $Log$
 * Revision 1.2  2000/11/15 16:14:52  bremner
 * committing patched version of zram source
 *
 * Revision 1.1  1998/04/15  08:47:29  marzetta
 * Initial revision
 *
 *
 */

#ifndef _VC_BASE_H_
#define _VC_BASE_H_
#include "z_base.h"

typedef int16 degreeT;		/* see next line */
#define MAXDEGREE MAXint16

/* degree = MARK_DEGREE = -1	point is marked
   degree = 0			point is inactive (all neighbours marked)
   degree = 1..MAXDEGREE	point is active
   */

extern int vc_NodeTag;		/* vc_init.c */
typedef struct vc_Node {
  Z_NODE_HEADER;		/* len; tag = vc_NodeTag */
  MINT vc_numPt;		/* total number of points */
  MINT vc_numAct;		/* number of active points (degree > 0) */
  MINT vc_numMarked;		/* number of inactive marked points */
  MINT vc_numEdge;		/* number of active edges */
  MINT vc_lb;			/* lower bound */
  degreeT vc_data[1];		/* variable size */
				/* vc_degree[]: degree of points */
} vc_Node, *vc_NodeP;

#define vc_degree(np)	(np->vc_data)
				/* active degree of points, mark */
			   /* the degree of an active point is redundant */
#define MARK_DEGREE	-1
#define vc_active(np, pt) (vc_degree(np)[pt] > 0)


extern int vc_GlobalTag;	/* vc_init.c */
typedef struct vc_Global {
  Z_NODE_HEADER;		/* len; tag = vc_GlobalTag */
  MINT vc_numPt;		/* total number of points */
  MINT vc_numAct;		/* number of active points (degree > 0) */
  MINT vc_numMarked;		/* number of inactive marked points */
  MINT vc_numEdge;		/* number of active edges */
  MINT vc_lbLeft;		/* lower bound of left subgraph */
  MINT vc_lbRight;		/* lower bound of right subgraph */
  MINT vc_procsetnum;		/* branch and bound procedure set */
  MINT vc_lbProc;		/* which lower bound procedure to use */
  MINT vc_gdata[1];		/* variable size */
				/* vc_gdegree[]: degree of points */
				/* vc_inleft[]: point is in left subgraph */
				/* vc_first[]: index into adjacency list */
				/*             valid if degree > 0 */
				/* vc_triangle[]: other point of triangle */
				/* vc_adj[]: adjacency lists */
} vc_Global, *vc_GlobalP;

#define vc_gdegree(gp)	(gp->vc_gdata)
#define vc_inleft(gp) (gp->vc_gdata + gp->vc_numPt)
#define vc_first(gp)	(assert(gp->tag == vc_GlobalTag), \
			 gp->vc_gdata + 2 * gp->vc_numPt)
				/* index into adjacency list */
#define vc_triangle(gp)	(gp->vc_gdata + 3 * gp->vc_numPt)
				/* other point of triangle */
#define vc_adj(gp)	(gp->vc_gdata + 4 * gp->vc_numPt)
				/* adjacency lists, terminated by ADJ_LAST */
#define vc_adjPt(gp,pt)	(vc_adj(gp) + vc_first(gp)[pt])
				/* adjacency list of point pt */
#define ADJ_LAST	-1
/* BetterQuickMatch() assumes that ADJ_LAST == -1 */
#define ADJ_SHIFT	(1 << 12)
#ifdef COMPLEX_EDGES
#define vc_pt(x)	(x % ADJ_SHIFT)	/* other point of this edge */
#define vc_cross(x)	(x / ADJ_SHIFT)/* index of inverse edge in vc_adj[] */
#define vc_cadj(pt,ed)	((pt) + (MINT)(ed) * ADJ_SHIFT)
#else /* COMPLEX_EDGES */
#define vc_pt(x)	(x)	/* other point of this edge */
#define vc_cross(x)	(-1)	/* index of inverse edge in vc_adj[] */
#define vc_cadj(pt,ed)	(pt)
#endif /* COMPLEX_EDGES */

/* An edge is active if both its endpoints are active. All procedures
   working with adjacency lists must check that.
   */

/* Procedures ------------------------------------------------------- */

vc_GlobalP vc_NewFullNode(MINT numpt, MINT numedge);
/* Allocate a full vc_Node. Initialize len, tag, vc_numPt = vc_numAct = pt,
   vc_numMarked = 0, vc_numEdge = numedge, vc_lbLeft = vc_lbRight = 0;
   */
  
vc_NodeP vc_NewShortNode(MINT numpt, MINT numedge);
/* Allocate a short vc_Node. Initialize len, tag, vc_numPt = vc_numAct = pt,
   vc_numMarked = 0, vc_numEdge = numedge, vc_lb = 0;
   */

void vc_FreeNode(vc_NodeP node);
/* Deallocate the node. */

/* All procedures with a vc_GlobalP and a vc_NodeP expect either a
   vc_GlobalP and a Short Node or NULL and a full node. */

void vc_PrintNode(vc_GlobalP global, vc_NodeP node);
/* Print node to stdout */

void vc_Mark(vc_GlobalP global, vc_NodeP np, MINT pt, MINT **degree1list);
/* Mark point pt and remove incident edges */
/* Save points of degree 1 in degree1list */

MINT vc_MaxDegreePoint(vc_NodeP np);
/* Return a point of maximum degree in np */

boolean vc_IsSolution(vc_NodeP np);
/* Is np a solution? */

int vc_CompareProc(vc_NodeP x, vc_NodeP y);
/* Compare the lower bounds of x and y */

char *vc_ShortNodeString(vc_NodeP np);
/* Return a static string describing np */

void MarkHeadTail(vc_GlobalP global, vc_NodeP np,
			 MINT **head, MINT **tail);
/* Mark partners of points in list. The points in the list have degree
   at most one. */

MINT SomeAdjacent(vc_GlobalP global, vc_NodeP np, MINT pt);
/* Return some point adjacent to pt. */

void UnmarkPoints(vc_GlobalP global, vc_NodeP np);
/* np has no active points. Unmark points if possible. */

void Restrict(vc_GlobalP global, vc_NodeP np, int left);
/* Restrict np to the left / right subgraph. Degree of points in the
   other subgraph is set to 0.
   */

typedef void (* TriangleProcP)(void *ref, int a, int b, int c);

void ForAllTriangles(vc_GlobalP global, vc_NodeP np,
		     TriangleProcP what, void *ref);
/* Try a partition of np into triangles.
   Call what(ref, a, b, c) for all triangles found.
   The triangles may intersect, but the first point (a) is different for
   all of them.
   */



#endif /* _VC_BASE_H_ */
